@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26521 $
+ * $Revision: 27446 $
  * $Author: rjongbloed $
- * $Date: 2011-10-04 23:57:06 -0500 (Tue, 04 Oct 2011) $
+ * $Date: 2012-04-16 11:26:36 -0500 (Mon, 16 Apr 2012) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -697,7 +697,10 @@ class H323H263PluginCapability : public H323VideoPluginCapability
     virtual PBoolean OnReceivedPDU(
       const H245_VideoCapability & pdu  /// PDU to get information from
     );
-    virtual PBoolean IsMatch(const PASN_Choice & subTypePDU) const;
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
+    ) const;
 };
 
 #endif // OPAL_VIDEO
