@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24807 $
+ * $Revision: 26728 $
  * $Author: rjongbloed $
- * $Date: 2010-10-19 23:33:34 -0500 (Tue, 19 Oct 2010) $
+ * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
  */
 
 #ifndef OPAL_H323_CHANNELS_H
@@ -330,7 +330,7 @@ class H323Channel : public PObject
     ) { paused = pause; }
   //@}
 
-    virtual void OnMediaCommand(OpalMediaCommand &);
+    virtual bool OnMediaCommand(const OpalMediaCommand &);
 
   protected:
     virtual void InternalClose();
