@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26149 $
+ * $Revision: 27080 $
  * $Author: rjongbloed $
- * $Date: 2011-07-10 08:08:41 -0500 (Sun, 10 Jul 2011) $
+ * $Date: 2012-03-01 01:05:32 -0600 (Thu, 01 Mar 2012) $
  */
 
 #ifndef OPAL_OPAL_MANAGER_H
@@ -593,19 +593,46 @@ class OpalManager : public PObject
     /**A call back function to monitor the progress of a transfer.
        When a transfer operation is initiated, the Transfer() function will
        generally return immediately and the transfer may take some time. This
-       call back can give indicateion to the application of the progress of
+       call back can give an indication to the application of the progress of
+       the transfer.
        the transfer.
 
-       The exact format of the \p info parameter is dependent on the protocol
-       being used. It will always have a value info["result"] which will be
-       "success" or "failed". Other variables are protocol dependent.
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
 
-       For SIP, there is an additional info["state"] containing the NOTIFY
-       subscription state and an info["code"] entry containing the 3 digit
-       code returned in the NOTIFY body.
+       An application can now make a decision during the transfer operation
+       to short circuit the sequence, or let it continue. It can also
+       determine if the transfer did not go through, and it should "take back"
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
 
        A return value of false will immediately disconnect the current call.
 
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
+
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
+
        The default behaviour returns false if info["result"] == "success".
       */
     virtual bool OnTransferNotify(
@@ -798,14 +825,6 @@ class OpalManager : public PObject
       PBoolean requiresPatchThread = true  ///< The patch requires a thread
     );
 
-    /**Destroy a OpalMediaPatch instance.
-
-       The default behaviour simply calls delete patch.
-      */
-    virtual void DestroyMediaPatch(
-      OpalMediaPatch * patch
-    );
-
     /**Call back for a media patch thread starting.
        This function is called within the context of the thread associated
        with the media patch.
@@ -861,6 +880,7 @@ class OpalManager : public PObject
 
   /**@name Other services */
   //@{
+    /// Message waiting sub-types
     enum MessageWaitingType { 
       NoMessageWaiting,
       VoiceMessageWaiting, 
@@ -871,7 +891,15 @@ class OpalManager : public PObject
       NumMessageWaitingTypes
     };
 
-    /**Callback called when Message Waiting Indication (MWI) is received
+    /**Callback called when Message Waiting Indication (MWI) is received.
+       Multiple callbacks may occur with each MessageWaitingType. A \p type
+       of NumMessageWaitingTypes indicates the server is unable to distinguish
+       the message type.
+
+       The \p extraInfo parameter is generally of the form "a/b" where a and b
+       unsigned integers representing new and old message count. However, it
+       may be a simple "yes" or "no" if the remote cannot provide a message
+       count.
      */
     virtual void OnMWIReceived(
       const PString & party,    ///< Name of party MWI is for
@@ -1587,6 +1615,7 @@ class OpalManager : public PObject
 
     PThread    * garbageCollector;
     PSyncPoint   garbageCollectExit;
+    bool         garbageCollectSkip;
     PDECLARE_NOTIFIER(PThread, OpalManager, GarbageMain);
 
 #ifdef OPAL_ZRTP
