@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26521 $
+ * $Revision: 27367 $
  * $Author: rjongbloed $
- * $Date: 2011-10-04 23:57:06 -0500 (Tue, 04 Oct 2011) $
+ * $Date: 2012-03-29 02:32:25 -0500 (Thu, 29 Mar 2012) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -313,21 +313,35 @@ class OpalMediaOption : public PObject
 
 #if OPAL_H323
     struct H245GenericInfo {
-      H245GenericInfo() { memset(this, 0, sizeof(*this)); }
-      unsigned ordinal:16;
+      H245GenericInfo()
+        : ordinal(0)
+        , mode(None)
+        , integerType(UnsignedInt)
+        , excludeTCS(false)
+        , excludeOLC(false)
+        , excludeReqMode(false)
+        , position(-1)
+      { }
+
+      unsigned ordinal;
+
       enum Modes {
         None,
         Collapsing,
         NonCollapsing
-      } mode:3;
+      } mode;
+
       enum IntegerTypes {
         UnsignedInt,
         Unsigned32,
         BooleanArray
-      } integerType:3;
-      bool excludeTCS:1;
-      bool excludeOLC:1;
-      bool excludeReqMode:1;
+      } integerType;
+
+      bool    excludeTCS;
+      bool    excludeOLC;
+      bool    excludeReqMode;
+      int     position;     // Position in sequence for parameter
+      PString defaultValue; // Do not include parameter if this value
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
@@ -711,9 +725,9 @@ class OpalMediaFormat : public PContainer
 {
     PCLASSINFO(OpalMediaFormat, PContainer)
   public:
-    OpalMediaFormat(const OpalMediaFormat & c) : PContainer(c), m_info(c.m_info) { }
+    OpalMediaFormat(const OpalMediaFormat & c);
+    virtual ~OpalMediaFormat();
     OpalMediaFormat & operator=(const OpalMediaFormat & c)     { AssignContents(c); return *this; }
-    virtual ~OpalMediaFormat()                                 { Destruct(); }
     virtual PBoolean MakeUnique();
   protected:
     virtual void DestroyContents();
@@ -1207,6 +1221,8 @@ class OpalMediaFormat : public PContainer
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
     static const PString & MediaPacketizationsOption();
+    PStringSet GetMediaPacketizations() const;
+    void SetMediaPacketizations(const PStringSet & packetizations);
 #endif
 
   private:
