@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23148 $
+ * $Revision: 27149 $
  * $Author: rjongbloed $
- * $Date: 2009-07-28 07:42:40 -0500 (Tue, 28 Jul 2009) $
+ * $Date: 2012-03-07 18:32:36 -0600 (Wed, 07 Mar 2012) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -53,6 +53,55 @@ class OpalConnection;
 class OpalRTPConnection;
 class OpalMediaStatistics;
 
+/*!< \page pageOpalMediaStream Media streams in the OPAL library
+
+\section secOverviewM Overview
+This page of the documentation is designed to provide an understanding of the
+use of the OpalMediaStream class in the OPAL library. 
+
+A voip call that is handled in OPAL consists of two instances of the
+OpalConnection class (or derivative) plus instances of the
+OpalMediaStream class (or derivatives). Consequently, for handling a
+H.323 call which is directed to the PC speaker/microphone, there will
+H323Connection and a PCSSConnection. OPAL requires that media can be
+transferred between these two clases. To transfer the media, OPAL uses
+the OpalMediaStream.
+
+Key to understanding the OpalMediaStream class is that this class is
+responsible for transferring media from one instance of an
+OpalConnection to another instance of an OpalConnection.
+
+\section secDescriptionM Description
+
+From \ref pageOpalConnections, we see that derivatives of the
+OpalConnection class take many different forms (depending on the
+protocol). Further, the media from those protocols may well have
+different forms. For example, H.323 and PCSS have quite different
+format for the media. The class OpalMediaStream (and derivatives) is
+responsible for making the format of the media (from each OpalConnection
+instance) consistent.
+
+The standard output format of an OpalMediaStream (or derivative) is a
+RTP_DataFrame. Consequently, different OpalConnection instances (or
+derivatives) can be connected together and exchange media (where the
+media is encapsulated inside instances of the RTP_DataFrame class).
+
+When handling an audio only call, each OpalConnection instance (or
+derivative) has two attached OpalMediaStream instances (or
+derivatives). When the call contains video and audio, each
+OpalConnection instance (or derivative) will have four attached
+OpalMediaStream instances (or derivatives).
+
+For handling a H.323 call, which is an audio only call, that goes the speaker/microphone of the box, there will be 
+\li one instance of a H323Connection
+\li one instance of a OpalPCSSConnection
+\li two instances of a OpalAudioMediaStream
+\li two instances of a OpalRTPMediaStream
+
+Note that an OpalMediaStream instance is designed to take media in one
+direction.
+
+*/
 
 /**This class describes a media stream as used in the OPAL system. A media
    stream is the channel through which media data is trasferred between OPAL
@@ -68,7 +117,7 @@ class OpalMediaStream : public PSafeObject
     /**Construct a new media stream.
       */
     OpalMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource                        ///<  Is a source stream
@@ -113,32 +162,25 @@ class OpalMediaStream : public PSafeObject
        The default behaviour updates the mediaFormat member variable and
        pases the value on to the OpalMediaPatch.
       */
-    virtual bool UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat,  ///<  New media format
-      bool fromPatch = false                ///<  Is being called from OpalMediaPatch
+    bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+    virtual bool InternalUpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
     );
 
     /**Execute the command specified to the transcoder. The commands are
-       highly context sensitive, for example VideoFastUpdate would only apply
-       to a video transcoder.
+       highly context sensitive, for example OpalVideoUpdatePicture would only
+       apply to a video transcoder.
 
        The default behaviour passes the command on to the OpalMediaPatch.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
     );
 
-    /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example VideoFastUpdate
-       would only apply to a video transcoder.
-
-       The default behaviour passes the command on to the OpalMediaPatch and
-       sets the member variable commandNotifier.
-      */
-    virtual void SetCommandNotifier(
-      const PNotifier & notifier    ///<  Command to execute.
-    );
-
     /**Open the media stream using the media format.
 
        The default behaviour simply sets the isOpen variable to true.
@@ -154,19 +196,22 @@ class OpalMediaStream : public PSafeObject
 
     /**Close the media stream.
 
-       The default does nothing.
+       The default marks the stream as closed and calls
+       OpalConnection::OnClosedMediaStream().
       */
     virtual PBoolean Close();
 
     /**Callback that is called on the source stream once the media patch has started.
        The default behaviour calls OpalConnection::OnMediaPatchStart()
       */
-    virtual void OnPatchStart();
+    virtual void OnStartMediaPatch();
 
     /**Callback that is called on the source stream once the media patch has started.
        The default behaviour calls OpalConnection::OnMediaPatchStop()
       */
-    virtual void OnPatchStop();
+    virtual void OnStopMediaPatch(
+      OpalMediaPatch & patch    ///< Media patch that is stopping
+    );
 
     /**Write a list of RTP frames of data to the sink media stream.
        The default behaviour simply calls WritePacket() on each of the
@@ -246,24 +291,26 @@ class OpalMediaStream : public PSafeObject
     virtual PBoolean IsSynchronous() const = 0;
 
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
-       This is called on the source stream and is passed the sink stream that the
-       patch will initially be using. The function could conditionally require
-       the patch thread to execute a thread reading and writing data, or prevent
-       it from doing so as it can do so in hardware in some way, e.g. if both
-       streams where on the same OpalLineInterfaceDevice.
+       This is called on the source/sink stream and is passed the sink/source
+       stream that the patch will initially be using. The function could
+       conditionally require the patch thread to execute a thread reading and
+       writing data, or prevent  it from doing so as it can do so in hardware
+       in some way, e.g. both streams are on the same OpalLineInterfaceDevice.
 
        The default behaviour simply returns true.
       */
     virtual PBoolean RequiresPatchThread(
-      OpalMediaStream * sinkStream  ///< Sink stream for this source
+      OpalMediaStream * stream  ///< Other stream in patch
     ) const;
     virtual PBoolean RequiresPatchThread() const; // For backward compatibility
 
     /**Enable jitter buffer for the media stream.
+       Returns true if a jitter buffer is enabled/disabled. Returns false if
+       no jitter buffer exists for the media stream.
 
-       The default behaviour does nothing.
+       The default behaviour does nothing and returns false.
       */
-    virtual void EnableJitterBuffer() const;
+    virtual bool EnableJitterBuffer(bool enab = true) const;
   //@}
 
   /**@name Member variable access */
@@ -284,6 +331,10 @@ class OpalMediaStream : public PSafeObject
      */
     unsigned GetSessionID() const { return sessionID; }
 
+    /**Get the session number of the stream.
+     */
+    void SetSessionID(unsigned id) { sessionID = id; }
+
     /**  Get the ID associated with this stream. Used for detecting two 
       *  the streams associated with a bidirectional media channel
       */
@@ -307,13 +358,15 @@ class OpalMediaStream : public PSafeObject
 
     /**Get the paused state for stream.
       */
-    bool IsPaused() const { return paused; }
+    bool IsPaused() const { return m_paused; }
 
     /**Set the paused state for stream.
        This will stop reading/writing data from the stream.
+       Returns true if the pause state was changed
       */
-    virtual void SetPaused(
-      bool pause    ///< Indicate that the stream should be paused
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
     );
 
     /**Returns true if the media stream is open.
@@ -326,25 +379,23 @@ class OpalMediaStream : public PSafeObject
       OpalMediaPatch * patch  ///<  Media patch thread
     );
 
-    /**Remove the patch thread that is using this stream.
-       This function is useful in case of streams which can be accessed by
-       multiple instances of OpalMediaPatch.
-
-       The default behaviour simply sets patchThread to NULL.
-    */
-    virtual void RemovePatch(OpalMediaPatch * patch);
-
     /**Get the patch thread that is using the stream.
       */
-    OpalMediaPatch * GetPatch() const { return mediaPatch; }
+    OpalMediaPatch * GetPatch() const { return m_mediaPatch; }
 
     /**Add a filter to the owning patch safely.
       */
-    void AddFilter(const PNotifier & Filter, const OpalMediaFormat & Stage =  OpalMediaFormat());
+    void AddFilter(
+      const PNotifier & filter,   ///< Filter notifier to be called.
+      const OpalMediaFormat & stage = OpalMediaFormat() ///< Stage in codec pipeline to call filter
+    ) const;
 
     /**Remove a filter from the owning patch safely.
       */
-    bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
+    bool RemoveFilter(
+      const PNotifier & filter,   ///< Filter notifier to be called.
+      const OpalMediaFormat & stage = OpalMediaFormat() ///< Stage in codec pipeline to call filter
+    ) const;
 
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
@@ -352,11 +403,21 @@ class OpalMediaStream : public PSafeObject
   //@}
 
   protected:
+    void IncrementTimestamp(PINDEX size);
+    bool InternalWriteData(const BYTE * data, PINDEX length, PINDEX & written);
+
+    /**Close any internal components of the stream.
+       This should be used in preference to overriding the Close() function as
+       it is guaranteed to be called exactly once and avoids race conditions in
+       the shut down sequence of a media stream.
+      */
+    virtual void InternalClose() = 0;
+
     OpalConnection & connection;
     unsigned         sessionID;
     PString          identifier;
     OpalMediaFormat  mediaFormat;
-    bool             paused;
+    bool             m_paused;
     bool             isSource;
     bool             isOpen;
     PINDEX           defaultDataSize;
@@ -364,8 +425,18 @@ class OpalMediaStream : public PSafeObject
     bool             marker;
     unsigned         mismatchedPayloadTypes;
 
-    OpalMediaPatch * mediaPatch;
-    PNotifier        commandNotifier;
+    typedef PSafePtr<OpalMediaPatch, PSafePtrMultiThreaded> PatchPtr;
+    PatchPtr m_mediaPatch;
+
+    RTP_DataFrame::PayloadTypes m_payloadType;
+    unsigned                    m_frameTime;
+    PINDEX                      m_frameSize;
+
+  private:
+    P_REMOVE_VIRTUAL_VOID(OnPatchStart());
+    P_REMOVE_VIRTUAL_VOID(OnPatchStop());
+    P_REMOVE_VIRTUAL_VOID(OnStopMediaPatch());
+    P_REMOVE_VIRTUAL_VOID(RemovePatch(OpalMediaPatch *));
 };
 
 typedef PSafePtr<OpalMediaStream> OpalMediaStreamPtr;
@@ -407,12 +478,20 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
     /**Construct a new media stream for RTP sessions.
       */
     OpalNullMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
       bool isSynchronous = false           ///<  Can accept data and block accordingly
     );
+    OpalNullMediaStream(
+      OpalConnection & conn,               ///<  Connection that owns the stream
+      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
+      unsigned sessionID,                  ///<  Session number for stream
+      bool isSource,                       ///<  Is a source stream
+      bool usePacingDelay,                 ///<  Use delay to pace stream I/O
+      bool requiresPatchThread             ///<  Requires a patch thread to execute
+    );
   //@}
 
   /**@name Overrides of OpalMediaStream class */
@@ -436,18 +515,21 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
     );
 	
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
-       The default behaviour returns false.
+       The default behaviour returns the value of m_isSynchronous.
       */
     virtual PBoolean RequiresPatchThread() const;
 
     /**Indicate if the media stream is synchronous.
-       Returns false.
+       Returns m_isSynchronous.
       */
     virtual PBoolean IsSynchronous() const;
   //@}
 
   protected:
-    bool           m_isSynchronous;
+    virtual void InternalClose() { }
+
+    bool m_isSynchronous;
+    bool m_requiresPatchThread;
 };
 
 
@@ -464,7 +546,7 @@ class OpalRTPMediaStream : public OpalMediaStream
        This will add a reference to the rtpSession passed in.
       */
     OpalRTPMediaStream(
-      OpalRTPConnection & conn,
+      OpalRTPConnection & conn,            ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       bool isSource,                       ///<  Is a source stream
       RTP_Session & rtpSession,            ///<  RTP session to stream to/from
@@ -486,11 +568,13 @@ class OpalRTPMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
+    /**Set the paused state for stream.
+       This will stop reading/writing data from the stream.
       */
-    virtual PBoolean Close();
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
+    );
 
     /**Read an RTP frame of data from the source media stream.
        The new behaviour simply calls RTP_Session::ReadData().
@@ -518,11 +602,29 @@ class OpalRTPMediaStream : public OpalMediaStream
       */
     virtual PBoolean IsSynchronous() const;
 
-    /**Enable jitter buffer for the media stream.
-
-       The default behaviour does nothing.
+    /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
+       The default behaviour dermines if the media will be flowing between two
+       RTP sessions within the same process. If so the
+       OpalRTPConnection::OnLocalRTP() is called, and if it returns true
+       indicating local handling then this function returns faklse to disable
+       the patch thread.
       */
-    virtual void EnableJitterBuffer() const;
+    virtual PBoolean RequiresPatchThread() const;
+
+    /**Enable jitter buffer for the media stream.
+       Returns true if a jitter buffer is enabled/disabled. Returns false if
+       no jitter buffer exists for the media stream.
+
+       The default behaviour sets the RTP_Session jitter buffer size according
+       to the connection parameters, then returns true.
+      */
+    virtual bool EnableJitterBuffer(bool enab = true) const;
+
+    /**Set the patch thread that is using this stream.
+      */
+    virtual PBoolean SetPatch(
+      OpalMediaPatch * patch  ///<  Media patch thread
+    );
 
     /** Return current RTP session
       */
@@ -535,6 +637,8 @@ class OpalRTPMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     RTP_Session & rtpSession;
     unsigned      minAudioJitterDelay;
     unsigned      maxAudioJitterDelay;
@@ -553,7 +657,7 @@ class OpalRawMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalRawMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -598,18 +702,14 @@ class OpalRawMediaStream : public OpalMediaStream
       bool autoDelete = true  ///< Auto delete channel on exit or replacement
     );
 
-    /**Close the media stream.
-
-       Closes the associated PChannel.
-      */
-    virtual PBoolean Close();
-
     /**Get average signal level in last frame.
       */
     virtual unsigned GetAverageSignalLevel();
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PChannel * m_channel;
     bool       m_autoDelete;
     PMutex     m_channelMutex;
@@ -636,7 +736,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
-      OpalConnection &,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -647,7 +747,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
-      OpalConnection & ,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -661,7 +761,6 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
        Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
-  //@}
 
     virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
@@ -677,6 +776,7 @@ class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPac
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
     );
+  //@}
 
   protected:
     PFile file;
@@ -699,11 +799,12 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
       PINDEX buffers,                      ///<  Number of buffers on sound channel
+      unsigned bufferTime,                 ///<  Buffering time on sound channel (milliseconds)
       PSoundChannel * channel,             ///<  Audio device to stream to/from
       bool autoDelete = true               ///<  Automatically delete PSoundChannel
     );
@@ -711,11 +812,12 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
       PINDEX buffers,                      ///<  Number of buffers on sound channel
+      unsigned bufferTime,                 ///<  Buffering time on sound channel (milliseconds)
       const PString & deviceName           ///<  Name of audio device to stream to/from
     );
   //@}
@@ -739,7 +841,8 @@ class OpalAudioMediaStream : public OpalRawMediaStream
   //@}
 
   protected:
-    PINDEX soundChannelBuffers;
+    PINDEX   m_soundChannelBuffers;
+    unsigned m_soundChannelBufferTime;
 };
 
 #endif // OPAL_PTLIB_AUDIO
@@ -761,12 +864,13 @@ class OpalVideoMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalVideoMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       PVideoInputDevice * inputDevice,     ///<  Device to use for video grabbing
       PVideoOutputDevice * outputDevice,   ///<  Device to use for video display
-      bool autoDelete = true               ///<  Automatically delete PVideoDevices
+      bool autoDeleteInput = true,         ///<  Automatically delete PVideoInputDevice
+      bool autoDeleteOutput = true         ///<  Automatically delete PVideoOutputDevice
     );
 
     /**Delete attached channel if autoDelete enabled.
@@ -776,6 +880,16 @@ class OpalVideoMediaStream : public OpalMediaStream
 
   /**@name Overrides of PChannel class */
   //@{
+    /**Update the media format. This can be used to adjust the
+       parameters of a codec at run time. Note you cannot change the basic
+       media format, eg change GSM0610 to G.711, only options for that
+       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
+       different then a OpalMediaFormat::Merge() is performed.
+      */
+    virtual bool InternalUpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+
     /**Open the media stream.
 
        The default behaviour sets the OpalLineInterfaceDevice format and
@@ -783,12 +897,6 @@ class OpalVideoMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
-      */
-    virtual PBoolean Close();
-
     /**Read raw media data from the source media stream.
        The default behaviour simply calls ReadPacket() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
@@ -825,23 +933,28 @@ class OpalVideoMediaStream : public OpalMediaStream
 
     /** Get the input device (e.g. for statistics)
       */
-    virtual PVideoInputDevice * GetVideoInputDevice() const {
-      return inputDevice;
+    virtual PVideoInputDevice * GetVideoInputDevice() const
+    {
+      return m_inputDevice;
     }
 
     /** Get the output device (e.g. for statistics)
       */
-    virtual PVideoOutputDevice * GetVideoOutputDevice() const {
-      return outputDevice;
+    virtual PVideoOutputDevice * GetVideoOutputDevice() const
+    {
+      return m_outputDevice;
     }
 
   //@}
 
   protected:
-    PVideoInputDevice  * inputDevice;
-    PVideoOutputDevice * outputDevice;
-    bool                 autoDelete;
-    PTimeInterval        lastGrabTime;
+    virtual void InternalClose();
+
+    PVideoInputDevice  * m_inputDevice;
+    PVideoOutputDevice * m_outputDevice;
+    bool                 m_autoDeleteInput;
+    bool                 m_autoDeleteOutput;
+    PTimeInterval        m_lastGrabTime;
 };
 
 #endif // OPAL_VIDEO
@@ -859,7 +972,7 @@ class OpalUDPMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalUDPMediaStream(
-      OpalConnection & conn,
+      OpalConnection & conn,               ///<  Connection that owns the stream
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
@@ -867,6 +980,8 @@ class OpalUDPMediaStream : public OpalMediaStream
     );
   //@}
 
+    ~OpalUDPMediaStream();
+
   /**@name Overrides of OpalMediaStream class */
   //@{
 
@@ -888,15 +1003,11 @@ class OpalUDPMediaStream : public OpalMediaStream
        Returns false.
       */
     virtual PBoolean IsSynchronous() const;
-
-    /**Close the media stream.
-       Closes the associated OpalTransportUDP.
-      */
-    virtual PBoolean Close();
-
   //@}
 
   private:
+    virtual void InternalClose();
+
     OpalTransportUDP & udpTransport;
 };
 
