@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25300 $
+ * $Revision: 27458 $
  * $Author: rjongbloed $
- * $Date: 2011-03-09 13:22:53 -0600 (Wed, 09 Mar 2011) $
+ * $Date: 2012-04-17 13:04:50 -0500 (Tue, 17 Apr 2012) $
  */
 
 #ifndef OPAL_OPAL_PATCH_H
@@ -57,7 +57,7 @@ class OpalTranscoder;
    the Start() function is called on the patch when the creator code is
    ready for it to begin. For example all sink streams have been added.
   */
-class OpalMediaPatch : public PObject
+class OpalMediaPatch : public PSafeObject
 {
     PCLASSINFO(OpalMediaPatch, PObject);
   public:
@@ -180,24 +180,14 @@ class OpalMediaPatch : public PObject
 
        The default behaviour passes the command on to the source or sinks
        and the sinks transcoders.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command,   ///<  Command to execute.
       PBoolean fromSink                       ///<  Flag for source or sink
     );
 
-    /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example OpalVideoUpdatePicture
-       would only apply to a video transcoder.
-
-       The default behaviour passes the command on to the source or sinks
-       and the sinks transcoders.
-      */
-    virtual void SetCommandNotifier(
-      const PNotifier & notifier,   ///<  Command to execute.
-      PBoolean fromSink                 ///<  Flag for source or sink
-    );
-
     /**Set streams to paused.
       */
     virtual bool SetPaused(
@@ -243,10 +233,13 @@ class OpalMediaPatch : public PObject
 #endif
   //@}
 
+    PDECLARE_NOTIFIER(OpalMediaCommand, OpalMediaPatch, OnMediaCommand);
+
   protected:
                 
     /**Called from the associated patch thread */
     virtual void Main();
+    void StopThread();
     bool DispatchFrame(RTP_DataFrame & frame);
     bool EnableJitterBuffer();
 
@@ -259,7 +252,6 @@ class OpalMediaPatch : public PObject
         ~Sink();
         bool UpdateMediaFormat(const OpalMediaFormat & mediaFormat);
         bool ExecuteCommand(const OpalMediaCommand & command);
-        void SetCommandNotifier(const PNotifier & notifier);
         bool WriteFrame(RTP_DataFrame & sourceFrame);
 #if OPAL_STATISTICS
         void GetStatistics(OpalMediaStatistics & statistics, bool fromSource) const;
@@ -294,10 +286,6 @@ class OpalMediaPatch : public PObject
     };
     PList<Filter> filters;
 
-#if OPAL_VIDEO
-    bool             m_videoDecoder;
-#endif
-    bool             m_bypassActive;
     OpalMediaPatch * m_bypassToPatch;
     OpalMediaPatch * m_bypassFromPatch;
     PSyncPoint       m_bypassEnded;
@@ -312,7 +300,6 @@ class OpalMediaPatch : public PObject
 
     Thread * patchThread;
     PMutex patchThreadMutex;
-    mutable PReadWriteMutex inUse;
 
   private:
     P_REMOVE_VIRTUAL(bool, OnPatchStart(), false);
